@@ -49,6 +49,12 @@ static RCoreHelpMessage help_msg_ic = {
 	NULL
 };
 
+static RCoreHelpMessage help_msg_iy = {
+	"Usage: iy", "", "Display type information",
+	"iy", "", "List types (structs, enums, function signatures)",
+	NULL
+};
+
 static RCoreHelpMessage help_msg_iz = {
 	"Usage: iz", "[][jq*]", "List strings",
 	"iz", "", "strings in data sections (in JSON/Base64)",
@@ -131,7 +137,7 @@ static RCoreHelpMessage help_msg_i = {
 	"iS", "[?]", "list sections, segments and compute their hash",
 	"it", "", "file hashes", // hashes in it? wtf, thats a pretty bad subcommand
 	"iT", "", "file signature", // iT for signatures omg thats worst
-	"iV", "", "display file version info", // wtf why not iv
+	"iv", "", "display file version info", // wtf why not iv
 	"iw", "", "show try/catch blocks", // bad naming..
 	"iz", "[?]", "strings in data sections (in JSON/Base64)",
 	NULL
@@ -202,23 +208,6 @@ static void classdump_keys(RCore *core, RBinObject *bo) {
 	}
 }
 
-static char *demangle_internal(RCore *core, int type, const char *s) {
-	char *res = NULL;
-	switch (type) {
-	case R_BIN_LANG_CXX: res = r_bin_demangle_cxx (core->bin->cur, s, 0); break;
-	case R_BIN_LANG_JAVA: res = r_bin_demangle_java (s); break;
-	case R_BIN_LANG_OBJC: res = r_bin_demangle_objc (NULL, s); break;
-	case R_BIN_LANG_SWIFT: res = r_bin_demangle_swift (s, core->bin->options.demangle_usecmd, core->bin->options.demangle_trylib); break;
-	case R_BIN_LANG_DLANG: res = r_bin_demangle_plugin (core->bin, "dlang", s); break;
-	case R_BIN_LANG_MSVC: res = r_bin_demangle_msvc (s); break;
-	case R_BIN_LANG_RUST: res = r_bin_demangle_rust (core->bin->cur, s, 0); break;
-	case R_BIN_LANG_PASCAL: res = r_bin_demangle_freepascal (s); break;
-	default:
-		r_core_return_value (core, 1);
-	}
-	return res;
-}
-
 static void cmd_info_demangle(RCore *core, const char *input, PJ *pj, int mode) {
 	if (input[1] == 'j' && input[2] == ' ') {
 		mode = R_MODE_JSON;
@@ -255,7 +244,24 @@ static void cmd_info_demangle(RCore *core, const char *input, PJ *pj, int mode)
 		}
 		r_core_return_value (core, 1);
 	}
-	char *res = demangle_internal (core, lang_type, text);
+	char *res = NULL;
+	{
+		// RBinDemangle requires an RBinFile to get the RBinOptions for the trylib option
+		// when there's no current file, we need to pass core->bin for proper config access
+		RBinFile *bf = NULL;
+		RBinFile *temp_bf = NULL;
+		if (core->bin) {
+			bf = core->bin->cur;
+		}
+		if (!bf && core->bin) {
+			// create a temporary wrapper to ensure demangle has access to config
+			temp_bf = R_NEW0 (RBinFile);
+			temp_bf->rbin = core->bin;
+			bf = temp_bf;
+		}
+		res = r_bin_demangle (bf, lang, text, 0, false);
+		free (temp_bf);
+	}
 	if (mode == R_MODE_JSON) {
 		pj_o (pj);
 		pj_ks (pj, "lang", lang);
@@ -846,6 +852,7 @@ static void cmd_ic_comma(RCore *core, const char *input) {
 	r_table_add_column (t, typeString, "type", 0);
 	r_table_add_column (t, typeString, "klass", 0);
 	r_table_add_column (t, typeString, "name", 0);
+	r_table_add_column (t, typeString, "origin", 0);
 	const bool iova = r_config_get_b (core->config, "io.va");
 	r_list_foreach (bfiles, objs_iter, bf) {
 		RBinObject *obj = bf->bo;
@@ -857,13 +864,15 @@ static void cmd_ic_comma(RCore *core, const char *input) {
 			RBinSymbol *method;
 			r_list_foreach (klass->methods, iter2, method) {
 				char *addr = r_str_newf ("0x%08"PFMT64x, iova? method->vaddr: method->paddr);
-				r_table_add_row (t, addr, "method", kname, method->name, NULL);
+				const char *origin_str = r_bin_class_origin_tostring (klass->origin);
+				r_table_add_row (t, addr, "method", kname, method->name, origin_str, NULL);
 				free (addr);
 			}
 			RBinField *field;
 			r_list_foreach (klass->fields, iter2, field) {
 				char *addr = r_str_newf ("0x%08"PFMT64x, iova? field->vaddr: field->paddr);
-				r_table_add_row (t, addr, "field", kname, field->name, NULL);
+				const char *origin_str = r_bin_class_origin_tostring (klass->origin);
+				r_table_add_row (t, addr, "field", kname, field->name, origin_str, NULL);
 				free (addr);
 			}
 		}
@@ -946,6 +955,7 @@ void cmd_ic_add(RCore *core, const char *input) {
 	if (!klass) {
 		klass = R_NEW0 (RBinClass);
 		klass->name = r_bin_name_new (klass_name);
+		klass->origin = R_BIN_CLASS_ORIGIN_USER;
 		r_list_append (klasses, klass);
 	}
 	if (method_name == NULL) {
@@ -1508,7 +1518,7 @@ static void cmd_iSm(RCore *core, const char *input, PJ **_pj, int mode, const bo
 	if (!bf) {
 		return;
 	}
-	RBinObject *bo = bf->bo;
+	RBinObject *bo = R_UNWRAP4 (core, bin, cur, bo);
 	if (!bo) {
 		return;
 	}
@@ -1838,81 +1848,94 @@ static void cmd_idp(RCore *core, PJ *pj, const char *input, bool is_array, int m
 	}
 }
 
+static void cmd_idl(RCore *core, const char *input) {
+	char *linkname = NULL;
+	RBinInfo *info = r_bin_get_info (core->bin);
+	if (info && info->dbglink) {
+		linkname = strdup (info->dbglink);
+		char *dot = (char *)r_str_lchr (linkname, '.');
+		if (dot) {
+			*dot = 0;
+		}
+	}
+	switch (input[2]) {
+	case '?':
+		r_core_cmd_help (core, help_msg_idl);
+		break;
+	case 0:
+		if (linkname) {
+			r_cons_println (core->cons, info->dbglink);
+		}
+		break;
+	case 'd': // "idld" // download debug info file from remote dwarf symbols server
+		if (linkname) {
+			char *url = r_str_newf ("%s/%s/debuginfo", r_config_get (core->config, "dbg.linkurl"), linkname);
+			if (input[3] == '*') {
+				r_cons_printf (core->cons, "%s\n", url);
+			} else {
+				char *dir_debuglink = strdup (r_config_get (core->config, "dir.debuglink"));
+				char *colon = strchr (dir_debuglink, ':');
+				if (colon) {
+					*colon = 0;
+				}
+				char *fullpath = r_str_newf ("%s/%s", dir_debuglink, info->dbglink);
+				if (r_file_exists (fullpath)) {
+					R_LOG_INFO ("Debuglink file already exists: %s", fullpath);
+				} else {
+					if (r_socket_http_download (url, NULL, fullpath)) {
+						R_LOG_INFO ("Downloaded debuglink file to %s", fullpath);
+					} else {
+						R_LOG_ERROR ("Failed to download debuglink file");
+					}
+				}
+				free (fullpath);
+				free (dir_debuglink);
+			}
+			free (url);
+		} else {
+			R_LOG_ERROR ("No debuglink file to download");
+		}
+		break;
+	case '*': // "idl*"
+		if (linkname) {
+			char *dirlink = strdup (r_config_get (core->config, "dir.debuglink"));
+			RList *paths = r_str_split_list (dirlink, ":", 0);
+			RListIter *iter;
+			bool found = false;
+			char *path;
+			r_list_foreach (paths, iter, path) {
+				char *f = r_str_newf ("%s/%s", path, info->dbglink);
+				if (r_file_exists (f)) {
+					found = true;
+					r_cons_printf (core->cons, "'obf %s\n", f);
+					free (f);
+					break;
+				}
+				free (f);
+			}
+			r_list_free (paths);
+			free (dirlink);
+			if (!found) {
+				R_LOG_ERROR ("Cannot find %s in dir.debuglink. Use idld instead", info->dbglink);
+				r_cons_printf (core->cons, "'obf %s\n", info->dbglink);
+			}
+		}
+		break;
+	default:
+		r_core_return_invalid_command (core, "idl", input[2]);
+		break;
+	}
+	free (linkname);
+}
+
 static void cmd_id(RCore *core, PJ *pj, const char *input, bool is_array, int mode) {
 	const bool va = r_config_get_b (core->config, "io.va");
 	switch (input[1]) {
+	case '?': // "id?"
+		r_core_cmd_help (core, help_msg_id);
+		break;
 	case 'l': // "idl"
-		if (input[2] == '?') {
-			r_core_cmd_help (core, help_msg_idl);
-		} else {
-			char *linkname = NULL;
-			RBinInfo *info = r_bin_get_info (core->bin);
-			if (info && info->dbglink) {
-				linkname = strdup (info->dbglink);
-				char *dot = (char *)r_str_lchr (linkname, '.');
-				if (dot) {
-					*dot = 0;
-				}
-			}
-			switch (input[2]) {
-			case 'd':
-				if (linkname) {
-					char *url = r_str_newf ("%s/%s/debuginfo", r_config_get (core->config, "dbg.linkurl"), linkname);
-					if (input[3] == '*') {
-						r_cons_printf (core->cons, "%s\n", url);
-					} else {
-						char *dir_debuglink = strdup (r_config_get (core->config, "dir.debuglink"));
-						char *colon = strchr (dir_debuglink, ':');
-						if (colon) {
-							*colon = 0;
-						}
-						// TODO: check if file exists before downloading
-						// TODO: use seprate path instead of the first one from the list?
-						R_LOG_WARN ("This curl oneliner is subject to command injection. Use it at your own risk");
-						r_sys_cmdf ("curl -o \"%s/%s\" \"%s\"", dir_debuglink, info->dbglink, url);
-						free (dir_debuglink);
-					}
-					free (url);
-				} else {
-					R_LOG_ERROR ("No debuglink file to download");
-				}
-				break;
-			case '*':
-				if (linkname) {
-					char *dirlink = strdup (r_config_get (core->config, "dir.debuglink"));
-					RList *paths = r_str_split_list (dirlink, ":", 0);
-					RListIter *iter;
-					bool found = false;
-					char *path;
-					r_list_foreach (paths, iter, path) {
-						char *f = r_str_newf ("%s/%s", path, info->dbglink);
-						if (r_file_exists (f)) {
-							found = true;
-							r_cons_printf (core->cons, "'obf %s\n", f);
-							free (f);
-							break;
-						}
-						free (f);
-					}
-					r_list_free (paths);
-					free (dirlink);
-					if (!found) {
-						R_LOG_ERROR ("Cannot find %s in dir.debuglink. Use idld instead", info->dbglink);
-						r_cons_printf (core->cons, "'obf %s\n", info->dbglink);
-					}
-				}
-				break;
-			case 0:
-				if (linkname) {
-					r_cons_println (core->cons, info->dbglink);
-				}
-				break;
-			default:
-				r_core_return_invalid_command (core, "idl", input[2]);
-				break;
-			}
-			free (linkname);
-		}
+		cmd_idl (core, input);
 		break;
 	case 'x': // "idx"
 		RBININFO ("source", R_CORE_BIN_ACC_SOURCE, NULL, 0);
@@ -1920,9 +1943,6 @@ static void cmd_id(RCore *core, PJ *pj, const char *input, bool is_array, int mo
 	case 'p': // "idp"
 		cmd_idp (core, pj, input, is_array, mode);
 		break;
-	case '?':
-		r_core_cmd_help (core, help_msg_id);
-		break;
 	case 0:
 	case '*':
 	case 'j':
@@ -2308,6 +2328,9 @@ static int cmd_info(void *data, const char *input) {
 		case 'c': // "ic?"
 			r_core_cmd_help (core, help_msg_ic);
 			break;
+		case 'y': // "iy?"
+			r_core_cmd_help (core, help_msg_iy);
+			break;
 		case 'd': // "id?"
 			r_core_cmd_help (core, help_msg_id);
 			break;
@@ -2580,6 +2603,15 @@ static int cmd_info(void *data, const char *input) {
 	case 'c': // "ic"
 		cmd_ic (core, input + 1, pj, is_array, va);
 		break;
+	case 'y': // "iy"
+		{
+			char *types = r_bin_get_types (core->bin);
+			if (types) {
+				r_cons_println (core->cons, types);
+				free (types);
+			}
+		}
+		break;
 	case 'D': // "iD"
 		cmd_info_demangle (core, input, pj, mode);
 		break;
